# ALMA Imaging Script

# Imaging, including Self-calibration, 
# for the HL Tau B4 Science-Verification Data

# This script was run interactively to find the optimal 
# clean iterations and mask, and then adapted such that
# it can be run automatically - a user can change
# the interactive option to True if they wish to 
# manually clean. Note the number of iterations is the *minimal*
# amount required to clean deeply enough.
# The self calibration combines all spectral windows and
# both polarizations for maximal signal-to-noise.

# Please check this script for various instructions
# to uncomment any commands if running cleaning interactively
# or if plots of the solutions want to be examined.

# NOTE - Ignore warnings about the mask from Tclean
#        these are not detrimental to the imaging in anyway.
#
#        Multiscale clean in tclean was also tested but the results
#        were essentially indistinguishable from the standard tclean 
#        and therefore standard tclean was used in this script.

thesteps = []
step_title = {0: 'Concatenate and Split HLTau',
              1: 'Cleaning the concatenated datafile - pre selfcal',
              2: 'pcal1 - Selfcal long time phase',
              3: 'pcal2 - Selfcal 4 min time phase',
              4: 'pcal3 - Selfcal inf - scan time phase',
              5: 'apcal - Selfcal long time amp solutions',
              6: 'split self-calibrated dataset',
              7: 'make all fits files'
          }


if 'applyonly' not in globals(): applyonly = False
try:
  print 'List of steps to be executed ...', mysteps
  thesteps = mysteps
except:
  print 'global variable mysteps not set.'
if (thesteps==[]):
  thesteps = range(0,len(step_title))
  print 'Executing all steps: ', thesteps

# The Python variable 'mysteps' will control which steps
# are executed when you start the script using
#   execfile('scriptForCalibration.py')
# e.g. setting
#   mysteps = [2,3,4]# before starting the script will make the script execute
# only steps 2, 3, and 4
# Setting mysteps = [] will make it execute all steps.

# WARNING- step = 0 will remove the calibrated.ms and HLTau_B4.ms files
# this means any selfcalibration or models saved will have to be run again
# in step sequence - i.e 0,1,2,3,...


import re
import os
import glob


if re.search('^4.7.2', casadef.casa_version) == None:
 sys.exit('ERROR: PLEASE USE THE SAME VERSION OF CASA THAT YOU USED FOR GENERATING THE SCRIPT: 4.7.2')


# This is the mask to fully cover the extent of HL Tau
# with the default imaging parameters
maskregion = 'poly[[67.90993995deg, 18.23277509deg], [67.90998253deg, 18.23278279deg], [67.91011431deg, 18.23277124deg], [67.91017108deg, 18.23274621deg], [67.91022177deg, 18.23269999deg], [67.91027042deg, 18.23265378deg], [67.91028867deg, 18.23260371deg], [67.91030286deg, 18.23253439deg], [67.91030084deg, 18.23244966deg], [67.91027651deg, 18.23238996deg], [67.91020149deg, 18.23233027deg], [67.91009201deg, 18.23235723deg], [67.91001700deg, 18.23239381deg], [67.90994401deg, 18.23245929deg], [67.90991765deg, 18.23252283deg], [67.90988116deg, 18.23259408deg], [67.90987305deg, 18.23266918deg], [67.90988319deg, 18.23270769deg], [67.90990346deg, 18.23275198deg]]'



# Concatenate all the calibrated data and then split HLTau
# with some slight averaging for faster imaging

mystep = 0
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]


  # Concat the datafiles - in time-taken order 
  os.system('rm -rf calibrated.ms*')

  concat(vis=['uid___A002_Xacdf75_X1389.ms.split.cal','uid___A002_Xacdf75_X15e2.ms.split.cal','uid___A002_Xace764_X7b8.ms.split.cal','uid___A002_Xace764_Xa09.ms.split.cal'], concatvis='calibrated.ms')

  # also split out only HLTau now
  # slight binning to make data smaller 
  
  os.system('rm -rf HLTau_B4.ms*')
  split(vis='calibrated.ms', outputvis='HLTau_B4.ms', datacolumn='data', field='HL Tau', width=4)


# Imaging the concatenated data - no selfcalibration has been done
mystep = 1
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  delmod(vis='HLTau_B4.ms', scr=True)
  os.system('rm -rf HLTau_B4_continuum.*')

  tclean(vis                = 'HLTau_B4.ms',
        imagename           = 'HLTau_B4_continuum',
        field               =   'HL Tau',
        specmode            =      'mfs',
        niter               =     14000 ,
        interactive         =       False ,
        imsize              = [2048, 2048] ,
        cell                = '0.007arcsec' ,
        weighting           =   'briggs',
        robust              = 0.0,
        mask                = maskregion,
        outframe            = 'LSRK',
        savemodel           = 'modelcolumn',  # or modelcolumn
        pbcor               = True)  


  # WARNING - if interactive = True and the cleaning is stopped
  #           using the red cross then the model will NOT be saved
  #           and therefore the following selfcalibration will
  #           not continue as expected
  #
  #           Please uncomment the below IF you are interactively
  #           cleaning to ensure the model is forcibly saved

 
 # tclean(vis                = 'HLTau_B4.ms',
 #       imagename           = 'HLTau_B4_continuum',
 #       field               =   'HL Tau',
 #       specmode            =      'mfs',
 #       niter               =     0 ,
 #       interactive         =       True ,
 #       imsize              = [2048, 2048] ,
 #       cell                = '0.007arcsec' ,
 #       weighting           =   'briggs',
 #       robust              = 0.0,
 #       calcres             = False,
 #       calcpsf             = False,       
 #       savemodel           = 'modelcolumn',  # to save for tclean
 #       pbcor               = True)  



# First long time phase selfcalibration - a single
# 4 solutions only, one for each dataset 
# to solve very long time offset to align long term phases
# this solves for any residual long term atmospherics or antenna position
# inaccuracies

mystep = 2
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]

  os.system('rm -rf pcal1*')
  gaincal(vis='HLTau_B4.ms', caltable='pcal1',  spw='', combine='spw,scan', calmode='p', solint='inf', refant='DV19', minsnr=3.0, minblperant=4, gaintype='T')

  # NOTE - uncomment if solutions want to be investigated
  #plotcal('pcal1', yaxis='phase', iteration='antenna')

  # apply the pcal1
  applycal(vis='HLTau_B4.ms', spwmap=[0,0,0,0], gaintable='pcal1')


  # Delete the model so a new one can be saved and re-image
  # with the selfcalibration applied (pcal1)
  delmod(vis='HLTau_B4.ms', scr=True)
  os.system('rm -rf HLTau_B4_continuum_pcal1.*')
  tclean(vis                = 'HLTau_B4.ms',
        imagename           = 'HLTau_B4_continuum_pcal1',
        field               =   'HL Tau',
        specmode            =      'mfs',
        niter               =     14000 ,
        interactive         =       False ,
        imsize              = [2048, 2048] ,
        cell                = '0.007arcsec' ,
        weighting           =   'briggs',
        robust              = 0.0,
        mask                = maskregion,
        savemodel           = 'modelcolumn',   # or model 
        pbcor               = True)  


  # WARNING - if interactive = True and the cleaning is stopped
  #           using the red cross then the model will NOT be saved
  #           and therefore the following selfcalibration will
  #           not continue as expected
  #
  #           Please uncomment the below IF you are interactively
  #           cleaning to ensure the model is forcibly saved

  #tclean(vis                = 'HLTau_B4.ms.pcal1',
  #      imagename           = 'HLTau_B4_continuum_pcal1',
  #      field               =   'HL Tau',
  #      specmode            =      'mfs',
  #      niter               =     0 ,
  #      interactive         =       True ,
  #      imsize              = [2048, 2048] ,
  #      cell                = '0.007arcsec' ,
  #      weighting           =   'briggs',
  #      robust              = 0.0,
  #      calcres             = False,
  #      calcpsf             = False,       
  #      savemodel           = 'modelcolumn',  # to save for tclean
  #      pbcor               = True)  




# Continue the self calibration by selecting a shorter time interval
# to begin to solve for any shorter term phase variations
mystep = 3
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]


  os.system('rm -rf pcal2*')
  gaincal(vis='HLTau_B4.ms', caltable='pcal2',  spw='', combine='spw,scan', calmode='p', solint='4min', refant='DV19', minsnr=3.0, minblperant=4, gaintype='T')


  # NOTE - uncomment if solutions want to be investigated
  #plotcal('pcal2', yaxis='phase', iteration='antenna')

  
  # WARNING - during the previous gain calibration a number of solutions are
  #           flagged. These are few, but on the longer baseline antennas
  #           and therefore carrying the flags in the applycal *WILL*
  #           marginally degrade the image resolution (>40 mas).
  #
  #           Flags are backed up in case they need to be restored to 
  #           their pre-self-calibration state

  applycal(vis='HLTau_B4.ms', spwmap=[0,0,0,0], gaintable='pcal2',flagbackup=True)

  # Delete the model so a new one can be saved, re-image with pcal2 applied
  delmod(vis='HLTau_B4.ms', scr=True)
  os.system('rm -rf HLTau_B4_continuum_pcal2.*')
  tclean(vis                = 'HLTau_B4.ms',
        imagename           = 'HLTau_B4_continuum_pcal2',
        field               =   'HL Tau',
        specmode            =      'mfs',
        niter               =     15000 ,
        interactive         =       False ,
        imsize              = [2048, 2048] ,
        cell                = '0.007arcsec' ,
        weighting           =   'briggs',
        robust              = 0.0,
        mask                = maskregion,
        savemodel           = 'modelcolumn', 
        pbcor               = True)  

  # WARNING - if interactive = True and the cleaning is stopped
  #           using the red cross then the model will NOT be saved
  #           and therefore the following selfcalibration will
  #           not continue as expected
  #
  #           Please uncomment the below IF you are interactively
  #           cleaning to ensure the model is forcibly saved

  #tclean(vis                = 'HLTau_B4.ms.pcal2',
  #      imagename           = 'HLTau_B4_continuum_pcal2',
  #      field               =   'HL Tau',
  #      specmode            =      'mfs',
  #      niter               =     0 ,
  #      interactive         =       True ,
  #      imsize              = [2048, 2048] ,
  #      cell                = '0.007arcsec' ,
  #      weighting           =   'briggs',
  #      robust              = 0.0,
  #      calcres             = False,
  #      calcpsf             = False,       
  #      savemodel           = 'modelcolumn',  # to save for tclean
  #      pbcor               = True)  


# Reduce the self calibration time down to 'inf'
# NOTE this is reaching the signal-to-noise limits of the data
mystep = 4
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]


  os.system('rm -rf pcal3*')
  gaincal(vis='HLTau_B4.ms', caltable='pcal3',  spw='', combine='spw', calmode='p', solint='inf', refant='DV19', minsnr=3.0, minblperant=4, gaintype='T')


  # NOTE - uncomment if solutions want to be investigated
  #plotcal('pcal3', yaxis='phase', iteration='antenna')
  
  # WARNING - during the previous gain calibration a number of solutions are
  #           flagged. These are few, but on the longer baseline antennas
  #           and therefore carrying the flags in the applycal *WILL*
  #           marginally degrade the image resolution (>45 mas).
  #
  #           Flags are backed up in case they need to be restored to 
  #           their pre-self-calibration state
  applycal(vis='HLTau_B4.ms', spwmap=[0,0,0,0], gaintable='pcal3',flagbackup=True)


  # Deleted the model so a new one can be saved 
  # and re-image with pcal3 applied
  delmod(vis='HLTau_B4.ms', scr=True)
  os.system('rm -rf HLTau_B4_continuum_pcal3.*')
  tclean(vis                = 'HLTau_B4.ms',
        imagename           = 'HLTau_B4_continuum_pcal3',
        field               =   'HL Tau',
        specmode            =      'mfs',
        niter               =     15000 ,
        interactive         =       False ,
        imsize              = [2048, 2048] ,
        cell                = '0.007arcsec' ,
        weighting           =   'briggs',
        robust              = 0.0,         
        mask                = maskregion,
        savemodel           = 'modelcolumn',  # or modelcolumn
        pbcor               = True)  


# Complete a long term amplitude self calibration
# one solution per dataset
mystep = 5
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]
  
  # NOTE - applying pcal3 while solving for the selfcalibration
  gaincal(vis='HLTau_B4.ms', caltable='apcal',  spw='', combine='spw,scan', calmode='ap', solint='inf', refant='DV19', minsnr=3.0, minblperant=4, gaintype='T', gaintable='pcal3', spwmap=[0,0,0,0])  

  # NOTE - uncomment if solutions want to be investigated
  #plotcal('apcal', yaxis='phase', iteration='antenna') 
  #plotcal('apcal', yaxis='amp', iteration='antenna') 

  # Phases are about zero 
  #(within a few degrees as expected given pcal3 is applied)
  # Amplitude gains are reasonable typically within 10%

 
  # Apply the final solutions pcal 3 and apcal
  # ensure spwmap for both is set
  applycal(vis='HLTau_B4.ms', spwmap=[[0,0,0,0],[0,0,0,0]], gaintable=['pcal3','apcal']) 

  # NOTE - optional check of the corrected amplitudes vs uvdistance
  #        to check for any odities after amplitude calibration.
  #        All appears corrected well in these data 
  #plotms(vis='HLTau_B4.ms',coloraxis='spw',yaxis='amp',avgchannel='16',  xaxis='uvdist',avgtime='100',ydatacolumn='corrected')


  # Delete the model so a one can be saved 
  # Make the pcal3 and apcal image
  delmod(vis='HLTau_B4.ms', scr=True)
  os.system('rm -rf HLTau_B4_continuum_pcal3_apcal.*')
  tclean(vis                = 'HLTau_B4.ms',
        imagename           = 'HLTau_B4_continuum_pcal3_apcal',
        field               =   'HL Tau',
        specmode            =      'mfs',
        niter               =     15000 ,
        interactive         =       False ,
        imsize              = [2048, 2048] ,
        cell                = '0.007arcsec' ,
        weighting           =   'briggs',
        robust              = 0.0,         
        mask                = maskregion,
        savemodel           = 'modelcolumn',  
        pbcor               = True)  


# Split the selfcalibrated dataset
mystep = 6
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]


  split(vis='HLTau_B4.ms', outputvis='HLTau_B4.ms.selfcalibrated')


# Make fits files of the images
mystep = 7
if(mystep in thesteps):
  casalog.post('Step '+str(mystep)+' '+step_title[mystep],'INFO')
  print 'Step ', mystep, step_title[mystep]



  # use glob
  fitslist=glob.glob('*image')
  for imagefits in fitslist:
    myimagebase=imagefits[:-6]
    print myimagebase
    # export the corrected image
    exportfits(imagename=myimagebase+'.image.pbcor', fitsimage=myimagebase+'.image.pbcor.fits') 
    # export the PB image
    exportfits(imagename=myimagebase+'.pb', fitsimage=myimagebase+'.flux.fits') 


